#include "mbed.h"
#include "SDFileSystem.h"
#include "mimic.h"
#include "fsdata.h"

/**
 * local filesystem support.
 */
LocalFileSystem lf("local");
SDFileSystem sd(p5, p6, p7, p8,"sd");

DigitalOut led1(LED1);
DigitalOut led3(LED3);
DigitalOut led4(LED4);

/**
 * MiMic RemoteMCU httpd.<br/>
 * <p>Service list</p>
 * <pre>
 * /rom/ - romfs
 * /setup/ - MiMic configulation REST API.
 * /local/ - mbed LocalFileSystem
 * /mvm/   - MiMicVM REST API
 * </pre>
 */
class MiMicRemoteMcu:public MiMic::Httpd
{
private:
    ModRomFiles modromfs; //ROM file module
    ModMiMicSetting mimicsetting; //mimic setting API
    ModRemoteMcu remotemcu; // remotemcu API
    ModLocalFileSystem modlocal; //basic URL parser
    ModLocalFileSystem modsd; //basic URL parser
    
public:
    MiMicRemoteMcu():Httpd(80)
    {
        this->modromfs.setParam("rom",RMCU_FSDATA,20);
        this->mimicsetting.setParam("setup");
        this->remotemcu.setParam("mvm");
        this->modlocal.setParam("local");
        this->modsd.setParam("sd");
    }
    virtual void onRequest(HttpdConnection& i_connection)
    {
        //try to ModRomFS module.
        if(this->modromfs.execute(i_connection)){
            return;
        }
        //try to ModMiMicSetting module.
        if(this->mimicsetting.execute(i_connection)){
            return;
        }
        //try to ModRemoteMcu module.
        if(this->remotemcu.execute(i_connection)){
            return;
        }
        //try to ModLocalFileSystem
        if(this->modlocal.execute(i_connection)){
            return;
        }
        //try to ModLocalFileSystem(SD)
        if(this->modsd.execute(i_connection)){
            return;
        }
        
        //Otherwise, Send the redirect response to /rom/index.html
        i_connection.sendHeader(301,
            "text/html",
            "Status: 302:Moved Temporarily\r\n"
            "Location: /rom/index.html\r\n");        
        return;
    }
};

int main()
{
    NetConfig cfg; //create network configulation  with onchip-setting.
    //try to override setting by local file.
    if(!cfg.loadFromFile("/local/mimic.cfg")){
        cfg.loadFromFile("/sd/mimic.cfg");
    }
    Net net(cfg);  //create a net instance
    MiMicRemoteMcu httpd; //create a httpd instance.
    httpd.loop();  //start httpd loop.
    return 0;
}
